@@ -2,7 +2,7 @@
  * This file is part of gspell, a spell-checking library.
  *
  * Copyright 2002-2006 - Paolo Maggi
- * Copyright 2015 - Sébastien Wilmet
+ * Copyright 2015, 2016 - Sébastien Wilmet
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -38,9 +38,9 @@
  * #GspellChecker is a spell checker. It is a wrapper around the Enchant
  * library, to have an API based on #GObject.
  *
- * If the #GspellChecker:language is not set correctly, the spell checker
- * should not be used. It can happen when no dictionaries are available, in
- * which case gspell_checker_get_language() returns %NULL.
+ * If the #GspellChecker:language property is %NULL, it means that no
+ * dictonaries are available, in which case the #GspellChecker is in a
+ * “disabled” (but allowed) state.
  */
 
 typedef struct _GspellCheckerPrivate GspellCheckerPrivate;
@@ -83,29 +83,6 @@ gspell_checker_error_quark (void)
 	return quark;
 }
 
-gboolean
-_gspell_checker_check_language_set (GspellChecker *checker)
-{
-	GspellCheckerPrivate *priv;
-
-	g_return_val_if_fail (GSPELL_IS_CHECKER (checker), FALSE);
-
-	priv = gspell_checker_get_instance_private (checker);
-
-	g_assert ((priv->active_lang == NULL && priv->dict == NULL) ||
-		  (priv->active_lang != NULL && priv->dict != NULL));
-
-	if (priv->active_lang == NULL)
-	{
-		g_warning ("Spell checker: the language is not correctly set.\n"
-			   "There is maybe no dictionaries available.\n"
-			   "Check the return value of gspell_checker_get_language().");
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
 static void
 gspell_checker_set_property (GObject      *object,
 			     guint         prop_id,
@@ -132,14 +109,12 @@ gspell_checker_get_property (GObject    *object,
 			     GValue     *value,
 			     GParamSpec *pspec)
 {
-	GspellCheckerPrivate *priv;
-
-	priv = gspell_checker_get_instance_private (GSPELL_CHECKER (object));
+	GspellChecker *checker = GSPELL_CHECKER (object);
 
 	switch (prop_id)
 	{
 		case PROP_LANGUAGE:
-			g_value_set_boxed (value, priv->active_lang);
+			g_value_set_boxed (value, gspell_checker_get_language (checker));
 			break;
 
 		default:
@@ -214,8 +189,8 @@ gspell_checker_class_init (GspellCheckerClass *klass)
 	 * @spell_checker: the #GspellChecker.
 	 * @word: the added word.
 	 *
-	 * Emitted when a word is added to the session dictionary. The session
-	 * dictionary is lost when the application exits.
+	 * Emitted when a word is added to the session dictionary. See
+	 * gspell_checker_add_word_to_session().
 	 */
 	signals[SIGNAL_WORD_ADDED_TO_SESSION] =
 		g_signal_new ("word-added-to-session",
@@ -259,8 +234,8 @@ gspell_checker_init (GspellChecker *checker)
  * gspell_checker_new:
  * @language: (nullable): the #GspellLanguage to use, or %NULL.
  *
- * Creates a new #GspellChecker. If @language is %NULL, finds the best available
- * language based on the current locale.
+ * Creates a new #GspellChecker. If @language is %NULL, the default language is
+ * picked with gspell_language_get_default().
  *
  * Returns: a new #GspellChecker object.
  */
@@ -272,137 +247,72 @@ gspell_checker_new (const GspellLanguage *language)
 			     NULL);
 }
 
-static const GspellLanguage *
-get_default_language (void)
-{
-	const GspellLanguage *lang;
-	const gchar * const *lang_names;
-	const GList *langs;
-	gint i;
-
-	/* Try with the current locale */
-	lang_names = g_get_language_names ();
-
-	for (i = 0; lang_names[i] != NULL; i++)
-	{
-		lang = gspell_language_lookup (lang_names[i]);
-
-		if (lang != NULL)
-		{
-			return lang;
-		}
-	}
-
-	/* Another try specific to Mac OS X */
-#ifdef OS_OSX
-	{
-		gchar *code = _gspell_osx_get_preferred_spell_language ();
-
-		if (code != NULL)
-		{
-			lang = gspell_language_lookup (code);
-			g_free (code);
-			return lang;
-		}
-	}
-#endif
-
-	/* Try English */
-	lang = gspell_language_lookup ("en_US");
-	if (lang != NULL)
-	{
-		return lang;
-	}
-
-	/* Take the first available language */
-	langs = gspell_language_get_available ();
-	if (langs != NULL)
-	{
-		return langs->data;
-	}
-
-	return NULL;
-}
-
-static gboolean
-init_dictionary (GspellChecker *checker)
+static void
+create_new_dictionary (GspellChecker *checker)
 {
 	GspellCheckerPrivate *priv;
+	const gchar *language_code;
 	const gchar *app_name;
 
 	priv = gspell_checker_get_instance_private (checker);
 
-	g_return_val_if_fail (priv->broker != NULL, FALSE);
-
-	if (priv->dict != NULL)
-	{
-		return TRUE;
-	}
-
-	if (priv->active_lang == NULL)
-	{
-		priv->active_lang = get_default_language ();
-	}
-
-	if (priv->active_lang != NULL)
-	{
-		const gchar *code;
-
-		code = gspell_language_get_code (priv->active_lang);
-
-		priv->dict = enchant_broker_request_dict (priv->broker, code);
-	}
-
-	if (priv->dict == NULL)
-	{
-		priv->active_lang = NULL;
-		return FALSE;
-	}
-
-	app_name = g_get_application_name ();
-	gspell_checker_add_word_to_session (checker, app_name, -1);
-
-	return TRUE;
-}
-
-/**
- * gspell_checker_set_language:
- * @checker: a #GspellChecker.
- * @language: (nullable): the #GspellLanguage to use, or %NULL.
- *
- * Sets the language to use for the spell checking. If @language is %NULL, finds
- * the best available language based on the current locale.
- *
- * Returns: whether the operation was successful.
- */
-gboolean
-gspell_checker_set_language (GspellChecker        *checker,
-			     const GspellLanguage *language)
-{
-	GspellCheckerPrivate *priv;
-	gboolean success;
-
-	g_return_val_if_fail (GSPELL_IS_CHECKER (checker), FALSE);
-
-	priv = gspell_checker_get_instance_private (checker);
-
-	if (language != NULL && priv->active_lang == language)
-	{
-		return TRUE;
-	}
-
 	if (priv->dict != NULL)
 	{
 		enchant_broker_free_dict (priv->broker, priv->dict);
 		priv->dict = NULL;
 	}
 
-	priv->active_lang = language;
-	success = init_dictionary (checker);
+	if (priv->active_lang == NULL)
+	{
+		return;
+	}
 
-	g_object_notify (G_OBJECT (checker), "language");
+	language_code = gspell_language_get_code (priv->active_lang);
+	priv->dict = enchant_broker_request_dict (priv->broker, language_code);
 
-	return success;
+	if (priv->dict == NULL)
+	{
+		/* Should never happen, no need to return a GError. */
+		g_warning ("Impossible to create an Enchant dictionary for the language code '%s'.",
+			   language_code);
+
+		priv->active_lang = NULL;
+		return;
+	}
+
+	app_name = g_get_application_name ();
+	gspell_checker_add_word_to_session (checker, app_name, -1);
+}
+
+/**
+ * gspell_checker_set_language:
+ * @checker: a #GspellChecker.
+ * @language: (nullable): the #GspellLanguage to use, or %NULL.
+ *
+ * Sets the language to use for the spell checking. If @language is %NULL, the
+ * default language is picked with gspell_language_get_default().
+ */
+void
+gspell_checker_set_language (GspellChecker        *checker,
+			     const GspellLanguage *language)
+{
+	GspellCheckerPrivate *priv;
+
+	g_return_if_fail (GSPELL_IS_CHECKER (checker));
+
+	priv = gspell_checker_get_instance_private (checker);
+
+	if (language == NULL)
+	{
+		language = gspell_language_get_default ();
+	}
+
+	if (priv->active_lang != language)
+	{
+		priv->active_lang = language;
+		create_new_dictionary (checker);
+		g_object_notify (G_OBJECT (checker), "language");
+	}
 }
 
 /**
@@ -431,7 +341,10 @@ gspell_checker_get_language (GspellChecker *checker)
  * @word_length: the byte length of @word, or -1 if @word is nul-terminated.
  * @error: (out) (optional): a location to a %NULL #GError, or %NULL.
  *
- * Returns: whether @word is correctly spelled.
+ * If the #GspellChecker:language is %NULL, i.e. when no dictonaries are
+ * available, this function returns %TRUE to limit the damage.
+ *
+ * Returns: %TRUE if @word is correctly spelled, %FALSE otherwise.
  */
 gboolean
 gspell_checker_check_word (GspellChecker  *checker,
@@ -448,12 +361,14 @@ gspell_checker_check_word (GspellChecker  *checker,
 	g_return_val_if_fail (word_length >= -1, FALSE);
 	g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
 
-	/* If no dictionaries are available, limit the damage by returning TRUE. */
-	g_return_val_if_fail (_gspell_checker_check_language_set (checker), TRUE);
-
 	priv = gspell_checker_get_instance_private (checker);
 
-	if (_gspell_utils_is_digit (word, word_length))
+	if (priv->dict == NULL)
+	{
+		return TRUE;
+	}
+
+	if (_gspell_utils_is_number (word, word_length))
 	{
 		return TRUE;
 	}
@@ -510,10 +425,14 @@ gspell_checker_get_suggestions (GspellChecker *checker,
 	g_return_val_if_fail (GSPELL_IS_CHECKER (checker), NULL);
 	g_return_val_if_fail (word != NULL, NULL);
 	g_return_val_if_fail (word_length >= -1, NULL);
-	g_return_val_if_fail (_gspell_checker_check_language_set (checker), NULL);
 
 	priv = gspell_checker_get_instance_private (checker);
 
+	if (priv->dict == NULL)
+	{
+		return NULL;
+	}
+
 	suggestions = enchant_dict_suggest (priv->dict, word, word_length, NULL);
 
 	if (suggestions == NULL)
@@ -526,7 +445,7 @@ gspell_checker_get_suggestions (GspellChecker *checker,
 		suggestions_list = g_slist_prepend (suggestions_list, suggestions[i]);
 	}
 
-	/* The single suggestions will be freed by the caller */
+	/* The array/list elements will be freed by the caller. */
 	g_free (suggestions);
 
 	return g_slist_reverse (suggestions_list);
@@ -538,7 +457,7 @@ gspell_checker_get_suggestions (GspellChecker *checker,
  * @word: a word.
  * @word_length: the byte length of @word, or -1 if @word is nul-terminated.
  *
- * Adds a word to the personal dictionary. It is typically saved in the user
+ * Adds a word to the personal dictionary. It is typically saved in the user's
  * home directory.
  */
 void
@@ -551,10 +470,14 @@ gspell_checker_add_word_to_personal (GspellChecker *checker,
 	g_return_if_fail (GSPELL_IS_CHECKER (checker));
 	g_return_if_fail (word != NULL);
 	g_return_if_fail (word_length >= -1);
-	g_return_if_fail (_gspell_checker_check_language_set (checker));
 
 	priv = gspell_checker_get_instance_private (checker);
 
+	if (priv->dict == NULL)
+	{
+		return;
+	}
+
 	enchant_dict_add (priv->dict, word, word_length);
 
 	if (word_length == -1)
@@ -581,9 +504,12 @@ gspell_checker_add_word_to_personal (GspellChecker *checker,
  * @word: a word.
  * @word_length: the byte length of @word, or -1 if @word is nul-terminated.
  *
- * Adds a word to the session dictionary. The session dictionary is lost when
- * the application exits. This function is typically called when an “Ignore All”
- * action is activated.
+ * Adds a word to the session dictionary. Each #GspellChecker instance has a
+ * different session dictionary. The session dictionary is lost when the
+ * #GspellChecker:language property changes or when @checker is destroyed or
+ * when gspell_checker_clear_session() is called.
+ *
+ * This function is typically called for an “Ignore All” action.
  */
 void
 gspell_checker_add_word_to_session (GspellChecker *checker,
@@ -595,10 +521,14 @@ gspell_checker_add_word_to_session (GspellChecker *checker,
 	g_return_if_fail (GSPELL_IS_CHECKER (checker));
 	g_return_if_fail (word != NULL);
 	g_return_if_fail (word_length >= -1);
-	g_return_if_fail (_gspell_checker_check_language_set (checker));
 
 	priv = gspell_checker_get_instance_private (checker);
 
+	if (priv->dict == NULL)
+	{
+		return;
+	}
+
 	enchant_dict_add_to_session (priv->dict, word, word_length);
 
 	if (word_length == -1)
@@ -628,19 +558,10 @@ gspell_checker_add_word_to_session (GspellChecker *checker,
 void
 gspell_checker_clear_session (GspellChecker *checker)
 {
-	GspellCheckerPrivate *priv;
-
 	g_return_if_fail (GSPELL_IS_CHECKER (checker));
-	g_return_if_fail (_gspell_checker_check_language_set (checker));
 
-	priv = gspell_checker_get_instance_private (checker);
-
-	/* free and re-request dictionary */
-	g_assert (priv->dict != NULL);
-	enchant_broker_free_dict (priv->broker, priv->dict);
-	priv->dict = NULL;
-
-	init_dictionary (checker);
+	/* Free and re-request dictionary. */
+	create_new_dictionary (checker);
 
 	g_signal_emit (G_OBJECT (checker), signals[SIGNAL_SESSION_CLEARED], 0);
 }
@@ -670,10 +591,14 @@ gspell_checker_set_correction (GspellChecker *checker,
 	g_return_if_fail (word_length >= -1);
 	g_return_if_fail (replacement != NULL);
 	g_return_if_fail (replacement_length >= -1);
-	g_return_if_fail (_gspell_checker_check_language_set (checker));
 
 	priv = gspell_checker_get_instance_private (checker);
 
+	if (priv->dict == NULL)
+	{
+		return;
+	}
+
 	enchant_dict_store_replacement (priv->dict,
 					word, word_length,
 					replacement, replacement_length);
