@@ -3,7 +3,7 @@
  *
  * Copyright 2006 - Paolo Maggi
  * Copyright 2008 - Novell, Inc.
- * Copyright 2015 - Sébastien Wilmet
+ * Copyright 2015, 2016 - Sébastien Wilmet
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,6 +25,20 @@
 #include <glib/gi18n-lib.h>
 #include <enchant.h>
 
+#ifdef OS_OSX
+#include "gspell-osx.h"
+#endif
+
+/**
+ * SECTION:language
+ * @Short_description: Language
+ * @Title: GspellLanguage
+ * @See_also: #GspellChecker
+ *
+ * #GspellLanguage represents a language that can be used for the spell
+ * checking, i.e. a language for which a dictionary is installed.
+ */
+
 #define ISO_639_DOMAIN	"iso_639"
 #define ISO_3166_DOMAIN	"iso_3166"
 
@@ -32,7 +46,15 @@ struct _GspellLanguage
 {
 	gchar *code;
 	gchar *name;
-	gchar *ckey;
+	gchar *collate_key;
+};
+
+typedef struct _DictsData DictsData;
+struct _DictsData
+{
+	GHashTable *iso_639_table;
+	GHashTable *iso_3166_table;
+	GTree *tree;
 };
 
 G_DEFINE_BOXED_TYPE (GspellLanguage,
@@ -40,15 +62,12 @@ G_DEFINE_BOXED_TYPE (GspellLanguage,
 		     gspell_language_copy,
 		     gspell_language_free)
 
-static GHashTable *iso_639_table = NULL;
-static GHashTable *iso_3166_table = NULL;
-
-#define ISOCODESLOCALEDIR ISO_CODES_PREFIX "/share/locale"
-
 #ifdef G_OS_WIN32
+
 #ifdef DATADIR
 #undef DATADIR
 #endif
+
 #include <shlobj.h>
 static HMODULE hmodule;
 
@@ -72,55 +91,38 @@ DllMain (HINSTANCE hinstDLL,
 	return TRUE;
 }
 
-static gchar *
-_get_iso_codes_prefix (void)
-{
-	static gchar retval[1000];
-	static gboolean beenhere = FALSE;
-	gchar *temp_dir = 0;
-
-	if (beenhere)
-	{
-		return retval;
-	}
-
-	if (!(temp_dir = g_win32_get_package_installation_directory_of_module ((gpointer) hmodule)))
-	{
-		strcpy (retval, ISO_CODES_PREFIX);
-		return retval;
-	}
-
-	strcpy (retval, temp_dir);
-	g_free (temp_dir);
-	beenhere = TRUE;
-	return retval;
-}
-
-static gchar *
-_get_isocodeslocaledir (void)
-{
-	static gchar retval[1000];
-	static gboolean beenhere = FALSE;
-
-	if (beenhere)
-	{
-		return retval;
-	}
-
-	strcpy (retval, _get_iso_codes_prefix ());
-	strcat (retval, "\\share\\locale");
-	beenhere = TRUE;
-	return retval;
-}
-
-#undef ISO_CODES_PREFIX
-#define ISO_CODES_PREFIX _get_iso_codes_prefix ()
-
-#undef ISOCODESLOCALEDIR
-#define ISOCODESLOCALEDIR _get_isocodeslocaledir ()
-
 #endif /* G_OS_WIN32 */
 
+static gchar *
+get_iso_codes_prefix (void)
+{
+	gchar *prefix = NULL;
+
+#ifdef G_OS_WIN32
+	prefix = g_win32_get_package_installation_directory_of_module ((gpointer) hmodule);
+#endif
+
+	if (prefix == NULL)
+	{
+		prefix = g_strdup (ISO_CODES_PREFIX);
+	}
+
+	return prefix;
+}
+
+static gchar *
+get_iso_codes_localedir (void)
+{
+	gchar *prefix;
+	gchar *localedir;
+
+	prefix = get_iso_codes_prefix ();
+	localedir = g_build_filename (prefix, "share", "locale", NULL);
+	g_free (prefix);
+
+	return localedir;
+}
+
 static void
 iso_639_start_element (GMarkupParseContext  *context,
 		       const gchar          *element_name,
@@ -134,33 +136,33 @@ iso_639_start_element (GMarkupParseContext  *context,
 	const gchar *iso_639_2_code = NULL;
 	const gchar *name = NULL;
 	const gchar *code = NULL;
-	gint ii;
+	gint i;
 
-	if (strcmp (element_name, "iso_639_entry") != 0)
+	if (g_strcmp0 (element_name, "iso_639_entry") != 0)
 	{
 		return;
 	}
 
-	for (ii = 0; attribute_names[ii] != NULL; ii++)
+	for (i = 0; attribute_names[i] != NULL; i++)
 	{
-		if (strcmp (attribute_names[ii], "name") == 0)
+		if (g_str_equal (attribute_names[i], "name"))
 		{
-			name = attribute_values[ii];
+			name = attribute_values[i];
 		}
-		else if (strcmp (attribute_names[ii], "iso_639_1_code") == 0)
+		else if (g_str_equal (attribute_names[i], "iso_639_1_code"))
 		{
-			iso_639_1_code = attribute_values[ii];
+			iso_639_1_code = attribute_values[i];
 		}
-		else if (strcmp (attribute_names[ii], "iso_639_2T_code") == 0)
+		else if (g_str_equal (attribute_names[i], "iso_639_2T_code"))
 		{
-			iso_639_2_code = attribute_values[ii];
+			iso_639_2_code = attribute_values[i];
 		}
 	}
 
 	code = (iso_639_1_code != NULL) ? iso_639_1_code : iso_639_2_code;
 
-	if (code != NULL && *code != '\0' &&
-	    name != NULL && *name != '\0')
+	if (code != NULL && code[0] != '\0' &&
+	    name != NULL && name[0] != '\0')
 	{
 		g_hash_table_insert (hash_table,
 				     g_strdup (code),
@@ -179,27 +181,27 @@ iso_3166_start_element (GMarkupParseContext  *context,
 	GHashTable *hash_table = data;
 	const gchar *name = NULL;
 	const gchar *code = NULL;
-	gint ii;
+	gint i;
 
-	if (strcmp (element_name, "iso_3166_entry") != 0)
+	if (g_strcmp0 (element_name, "iso_3166_entry") != 0)
 	{
 		return;
 	}
 
-	for (ii = 0; attribute_names[ii] != NULL; ii++)
+	for (i = 0; attribute_names[i] != NULL; i++)
 	{
-		if (strcmp (attribute_names[ii], "name") == 0)
+		if (g_str_equal (attribute_names[i], "name"))
 		{
-			name = attribute_values[ii];
+			name = attribute_values[i];
 		}
-		else if (strcmp (attribute_names[ii], "alpha_2_code") == 0)
+		else if (g_str_equal (attribute_names[i], "alpha_2_code"))
 		{
-			code = attribute_values[ii];
+			code = attribute_values[i];
 		}
 	}
 
-	if (code != NULL && *code != '\0' &&
-	    name != NULL && *name != '\0')
+	if (code != NULL && code[0] != '\0' &&
+	    name != NULL && name[0] != '\0')
 	{
 		g_hash_table_insert (hash_table,
 				     g_ascii_strdown (code, -1),
@@ -207,33 +209,24 @@ iso_3166_start_element (GMarkupParseContext  *context,
 	}
 }
 
-static GMarkupParser iso_639_parser =
-{
-	iso_639_start_element,
-	NULL, NULL, NULL, NULL
-};
-
-static GMarkupParser iso_3166_parser =
-{
-	iso_3166_start_element,
-	NULL, NULL, NULL, NULL
-};
-
 static void
 iso_codes_parse (const GMarkupParser *parser,
 		 const gchar         *basename,
 		 GHashTable          *hash_table)
 {
 	GMappedFile *mapped_file;
+	gchar *prefix;
 	gchar *filename;
 	GError *error = NULL;
 
-	filename = g_build_filename (ISO_CODES_PREFIX,
+	prefix = get_iso_codes_prefix ();
+	filename = g_build_filename (prefix,
 				     "share",
 				     "xml",
 				     "iso-codes",
 				     basename,
 				     NULL);
+	g_free (prefix);
 
 	mapped_file = g_mapped_file_new (filename, FALSE, &error);
 	g_free (filename);
@@ -264,7 +257,7 @@ spell_language_dict_describe_cb (const gchar * const language_code,
                                  const gchar * const provider_name,
                                  const gchar * const provider_desc,
                                  const gchar * const provider_file,
-                                 GTree *tree)
+				 DictsData *data)
 {
 	const gchar *iso_639_name;
 	const gchar *iso_3166_name;
@@ -279,7 +272,7 @@ spell_language_dict_describe_cb (const gchar * const language_code,
 
 	g_return_if_fail (tokens != NULL);
 
-	iso_639_name = g_hash_table_lookup (iso_639_table, tokens[0]);
+	iso_639_name = g_hash_table_lookup (data->iso_639_table, tokens[0]);
 
 	if (iso_639_name == NULL)
 	{
@@ -294,7 +287,7 @@ spell_language_dict_describe_cb (const gchar * const language_code,
 		goto exit;
 	}
 
-	iso_3166_name = g_hash_table_lookup (iso_3166_table, tokens[1]);
+	iso_3166_name = g_hash_table_lookup (data->iso_3166_table, tokens[1]);
 
 	if (iso_3166_name != NULL)
 	{
@@ -318,7 +311,7 @@ spell_language_dict_describe_cb (const gchar * const language_code,
 exit:
 	g_strfreev (tokens);
 
-	g_tree_replace (tree, g_strdup (language_code), language_name);
+	g_tree_replace (data->tree, g_strdup (language_code), language_name);
 }
 
 static gboolean
@@ -331,7 +324,7 @@ spell_language_traverse_cb (const gchar  *code,
 	language = g_slice_new (GspellLanguage);
 	language->code = g_strdup (code);
 	language->name = g_strdup (name);
-	language->ckey = g_utf8_collate_key (name, -1);
+	language->collate_key = g_utf8_collate_key (name, -1);
 
 	*available_languages = g_list_insert_sorted (*available_languages,
 						     language,
@@ -344,15 +337,28 @@ spell_language_traverse_cb (const gchar  *code,
  * gspell_language_get_available:
  *
  * Returns: (transfer none) (element-type GspellLanguage): the list of available
- * languages for the spell checking.
+ * languages, sorted with gspell_language_compare().
  */
 const GList *
 gspell_language_get_available (void)
 {
 	static gboolean initialized = FALSE;
 	static GList *available_languages = NULL;
+	gchar *localedir;
 	EnchantBroker *broker;
-	GTree *tree;
+	DictsData data;
+
+	GMarkupParser iso_639_parser =
+	{
+		iso_639_start_element,
+		NULL, NULL, NULL, NULL
+	};
+
+	GMarkupParser iso_3166_parser =
+	{
+		iso_3166_start_element,
+		NULL, NULL, NULL, NULL
+	};
 
 	if (initialized)
 	{
@@ -361,58 +367,136 @@ gspell_language_get_available (void)
 
 	initialized = TRUE;
 
-	bindtextdomain (ISO_639_DOMAIN, ISOCODESLOCALEDIR);
+	localedir = get_iso_codes_localedir ();
+
+	bindtextdomain (ISO_639_DOMAIN, localedir);
 	bind_textdomain_codeset (ISO_639_DOMAIN, "UTF-8");
 
-	bindtextdomain (ISO_3166_DOMAIN, ISOCODESLOCALEDIR);
+	bindtextdomain (ISO_3166_DOMAIN, localedir);
 	bind_textdomain_codeset (ISO_3166_DOMAIN, "UTF-8");
 
-	iso_639_table = g_hash_table_new_full (g_str_hash,
-					       g_str_equal,
-					       (GDestroyNotify) g_free,
-					       (GDestroyNotify) g_free);
+	g_free (localedir);
 
-	iso_3166_table = g_hash_table_new_full (g_str_hash,
-						g_str_equal,
-						(GDestroyNotify) g_free,
-						(GDestroyNotify) g_free);
+	data.iso_639_table = g_hash_table_new_full (g_str_hash,
+						    g_str_equal,
+						    (GDestroyNotify) g_free,
+						    (GDestroyNotify) g_free);
 
-	iso_codes_parse (&iso_639_parser, "iso_639.xml", iso_639_table);
-	iso_codes_parse (&iso_3166_parser, "iso_3166.xml", iso_3166_table);
+	data.iso_3166_table = g_hash_table_new_full (g_str_hash,
+						     g_str_equal,
+						     (GDestroyNotify) g_free,
+						     (GDestroyNotify) g_free);
 
-	tree = g_tree_new_full ((GCompareDataFunc) strcmp,
-				NULL,
-				(GDestroyNotify) g_free,
-				(GDestroyNotify) g_free);
+	iso_codes_parse (&iso_639_parser, "iso_639.xml", data.iso_639_table);
+	iso_codes_parse (&iso_3166_parser, "iso_3166.xml", data.iso_3166_table);
+
+	data.tree = g_tree_new_full ((GCompareDataFunc) strcmp,
+				     NULL,
+				     (GDestroyNotify) g_free,
+				     (GDestroyNotify) g_free);
 
 	broker = enchant_broker_init ();
 	enchant_broker_list_dicts (broker,
 				   (EnchantDictDescribeFn) spell_language_dict_describe_cb,
-				   tree);
+				   &data);
 	enchant_broker_free (broker);
 
-	g_tree_foreach (tree,
+	g_tree_foreach (data.tree,
 			(GTraverseFunc) spell_language_traverse_cb,
 			&available_languages);
 
-	g_tree_destroy (tree);
+	g_hash_table_unref (data.iso_639_table);
+	g_hash_table_unref (data.iso_3166_table);
+	g_tree_unref (data.tree);
 
 	return available_languages;
 }
 
+/**
+ * gspell_language_get_default:
+ *
+ * Finds the best available language based on the current locale.
+ *
+ * Returns: (nullable): the default #GspellLanguage, or %NULL if no dictionaries
+ * are available.
+ */
+const GspellLanguage *
+gspell_language_get_default (void)
+{
+	const GspellLanguage *lang;
+	const gchar * const *lang_names;
+	const GList *langs;
+	gint i;
+
+	/* Try with the current locale */
+	lang_names = g_get_language_names ();
+
+	for (i = 0; lang_names[i] != NULL; i++)
+	{
+		lang = gspell_language_lookup (lang_names[i]);
+
+		if (lang != NULL)
+		{
+			return lang;
+		}
+	}
+
+	/* Another try specific to Mac OS X */
+#ifdef OS_OSX
+	{
+		gchar *code = _gspell_osx_get_preferred_spell_language ();
+
+		if (code != NULL)
+		{
+			lang = gspell_language_lookup (code);
+			g_free (code);
+
+			if (lang != NULL)
+			{
+				return lang;
+			}
+		}
+	}
+#endif
+
+	/* Try English */
+	lang = gspell_language_lookup ("en_US");
+	if (lang != NULL)
+	{
+		return lang;
+	}
+
+	/* Take the first available language */
+	langs = gspell_language_get_available ();
+	if (langs != NULL)
+	{
+		return langs->data;
+	}
+
+	return NULL;
+}
+
+/**
+ * gspell_language_lookup:
+ * @language_code: a language code.
+ *
+ * Returns: (nullable): a #GspellLanguage corresponding to @language_code, or
+ * %NULL if not found.
+ */
 const GspellLanguage *
 gspell_language_lookup (const gchar *language_code)
 {
 	const GspellLanguage *closest_match = NULL;
 	const GList *available_languages;
+	const GList *l;
 
 	g_return_val_if_fail (language_code != NULL, NULL);
 
 	available_languages = gspell_language_get_available ();
 
-	while (available_languages != NULL)
+	for (l = available_languages; l != NULL; l = l->next)
 	{
-		GspellLanguage *language = available_languages->data;
+		const GspellLanguage *language = l->data;
 		const gchar *code = language->code;
 		gsize length = strlen (code);
 
@@ -425,13 +509,17 @@ gspell_language_lookup (const gchar *language_code)
 		{
 			closest_match = language;
 		}
-
-		available_languages = g_list_next (available_languages);
 	}
 
 	return closest_match;
 }
 
+/**
+ * gspell_language_get_code:
+ * @language: a #GspellLanguage.
+ *
+ * Returns: the @language code, for example fr_BE.
+ */
 const gchar *
 gspell_language_get_code (const GspellLanguage *language)
 {
@@ -440,20 +528,35 @@ gspell_language_get_code (const GspellLanguage *language)
 	return language->code;
 }
 
+/**
+ * gspell_language_get_name:
+ * @language: a #GspellLanguage.
+ *
+ * Returns the @language name translated to the current locale. For example
+ * "French (Belgium)" is returned if the current locale is in English and the
+ * @language code is fr_BE.
+ *
+ * Returns: the @language name.
+ */
 const gchar *
 gspell_language_get_name (const GspellLanguage *language)
 {
-	if (language == NULL)
-	{
-		/* Translators: This refers to the default language used by the
-		 * spell checker.
-		 */
-		return C_("language", "Default");
-	}
+	g_return_val_if_fail (language != NULL, NULL);
 
 	return language->name;
 }
 
+/**
+ * gspell_language_compare:
+ * @language_a: a #GspellLanguage.
+ * @language_b: another #GspellLanguage.
+ *
+ * Compares alphabetically two languages by their name, as returned by
+ * gspell_language_get_name().
+ *
+ * Returns: an integer less than, equal to, or greater than zero, if @language_a
+ * is <, == or > than @language_b.
+ */
 gint
 gspell_language_compare (const GspellLanguage *language_a,
                          const GspellLanguage *language_b)
@@ -461,7 +564,7 @@ gspell_language_compare (const GspellLanguage *language_a,
 	g_return_val_if_fail (language_a != NULL, 0);
 	g_return_val_if_fail (language_b != NULL, 0);
 
-	return g_strcmp0 (language_a->ckey, language_b->ckey);
+	return g_strcmp0 (language_a->collate_key, language_b->collate_key);
 }
 
 /**
